# ==== LongTable Class

#' [[ Method for LongTable Class
#'
#' Select an assay from within a LongTable object.
#'
#' @describeIn LongTable Get an assay from a LongTable object. This method
#'   returns the row and column annotations by default to make assignment
#'   and aggregate operations easiers.
#'
#' @examples
#' merckLongTable[['sensitivity']]
#'
#' @param x `LongTable` object to retrieve assays from
#' @param i `character` name or `integer` index of the desired assay.
#' @param withDimnames `logical` Should the row and column IDs be joined to
#'    the assay. Default is TRUE to allow easy use of group by arguments when
#'    performing data aggregation using the `data.table` API.
#' @param metadata `logical` Should the row and column metadata also
#'    be joined to the to the returned assay. Default is withDimnames.
#' @param keys `logical` Should the row and column keys also be returned?
#'    Defaults to !withDimnames.
#'
#' @importFrom crayon cyan magenta
#' @import data.table
#' @export
setMethod('[[', signature('LongTable'),
    function(x, i, withDimnames=TRUE, metadata=withDimnames, keys=!withDimnames) 
{
    funContext <- .S4MethodContext('[[', class(x))

    if (metadata && !withDimnames) {
        .warning('\nUnable to return metadata without dimnames, proceeding',
            ' as if withDimnames=TRUE.')
        withDimnames <- TRUE
    }

    if (length(i) > 1)
        .error('\nPlease only select one assay! To subset on multiple',
            'assays please see ?subset')

    if (keys) {
        .warning('\nIgnoring withDimnames and metadata arguments when',
            ' keys=TRUE.')
        return(assay(x, i))
    } else {
        if (withDimnames || metadata)
            return(assay(x, i, withDimnames, metadata))
        else
            return(assay(x, i)[, -c('rowKey', 'colKey')])
    }
})

#' `[[<-` Method for LongTable Class
#'
#' Just a wrapper around assay<- for convenience. See 
#' `?'assay<-,LongTable,character-method'.`
#'
#' @param x A `LongTable` to update.
#' @param i The name of the assay to update, must be in `assayNames(object)`.
#' @param value A `data.frame`
#'
#' @examples
#' merckLongTable[['sensitivity']] <- merckLongTable[['sensitivity']]
#'
#' @return A `LongTable` object with the assay `i` updated using `value`.
#'
#' @export
setReplaceMethod('[[', signature(x='LongTable'), 
    function(x, i, value) 
{
    assay(x, i) <- value
})